# Generated by Django 3.2.16 on 2023-01-23 04:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptionmanager', '0002_changesummary_chargesummary_paymentmethodsummary_recurringcreditsummary_revenueforecastsummary_subsc'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chargesummary',
            name='total',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
        migrations.AlterField(
            model_name='xeroinvoicesummary',
            name='is_sent',
            field=models.BooleanField(default=False, null=True),
        ),
    ]
