# Generated by Django 3.2.16 on 2023-04-12 00:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workforce', '0003_serverusage_contractor_addons_used'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomReportSummary',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tenant', models.CharField(db_index=True, max_length=200)),
                ('last_synced', models.DateTimeField(db_index=True)),
                ('custom_report_pk', models.PositiveIntegerField(db_index=True)),
                ('query', models.TextField()),
            ],
            options={
                'db_table': 'workforce"."customer_custom_reports',
                'managed': True,
                'default_permissions': (),
            },
        ),
    ]
